@@ -464,6 +464,25 @@ int main(int argc, char *argv[])
 	return 0;
 }
 
+/*
+ * Safe read (for pipes)
+ */
+
+ssize_t safe_read(int fd, void *buf, size_t count)
+{
+	ssize_t result = 0, res;
+
+	while (count > 0) {
+		res = read(fd, buf, count);
+		if (res < 0)
+			return result > 0 ? result : res;
+		count -= res;
+		result += res;
+		(char *)buf += res;
+	}
+	return result;
+}
+
 /*
  * Test, if it is a .VOC file and return >=0 if ok (this is the length of rest)
  *                                       < 0 if not 
@@ -572,7 +591,7 @@ static int test_au(int fd, void *buffer)
 	format.voices = BE_INT(ap->channels);
 	if (format.voices < 1 || format.voices > 128)
 		return -1;
-	if (read(fd, buffer + sizeof(AuHeader), BE_INT(ap->hdr_size) - sizeof(AuHeader)) < 0) {
+	if (safe_read(fd, buffer + sizeof(AuHeader), BE_INT(ap->hdr_size) - sizeof(AuHeader)) < 0) {
 		fprintf(stderr, "%s: read error\n", command);
 		exit(1);
 	}
@@ -731,14 +750,14 @@ static void voc_play(int fd, int ofs, char *name)
 	}
 	/* first we waste the rest of header, ugly but we don't need seek */
 	while (ofs > buffer_size) {
-		if (read(fd, audiobuf, buffer_size) != buffer_size) {
+		if (safe_read(fd, audiobuf, buffer_size) != buffer_size) {
 			fprintf(stderr, "%s: read error\n", command);
 			exit(1);
 		}
 		ofs -= buffer_size;
 	}
 	if (ofs)
-		if (read(fd, audiobuf, ofs) != ofs) {
+		if (safe_read(fd, audiobuf, ofs) != ofs) {
 			fprintf(stderr, "%s: read error\n", command);
 			exit(1);
 		}
@@ -756,7 +775,7 @@ static void voc_play(int fd, int ofs, char *name)
 			if (in_buffer)
 				memcpy(audiobuf, data, in_buffer);
 			data = audiobuf;
-			if ((l = read(fd, audiobuf + in_buffer, buffer_size - in_buffer)) > 0)
+			if ((l = safe_read(fd, audiobuf + in_buffer, buffer_size - in_buffer)) > 0)
 				in_buffer += l;
 			else if (!in_buffer) {
 				/* the file is truncated, so simulate 'Terminator' 
@@ -1214,7 +1233,7 @@ void playback_go(int fd, int loaded, u_long count, int rtype, char *name)
 			if (c + l > buffer_size)
 				c = buffer_size - l;
 				
-			if ((r = read(fd, audiobuf + l, c)) <= 0)
+			if ((r = safe_read(fd, audiobuf + l, c)) <= 0)
 				break;
 			l += r;
 		} while (mode != SND_PCM_MODE_STREAM || l < buffer_size);
@@ -1380,7 +1399,7 @@ static void playback(char *name)
 		}
 	}
 	/* read the file header */
-	if (read(fd, audiobuf, sizeof(AuHeader)) != sizeof(AuHeader)) {
+	if (safe_read(fd, audiobuf, sizeof(AuHeader)) != sizeof(AuHeader)) {
 		fprintf(stderr, "%s: read error", command);
 		exit(1);
 	}
@@ -1389,7 +1408,7 @@ static void playback(char *name)
 		playback_go(fd, 0, count, FORMAT_AU, name);
 		goto __end;
 	}
-	if (read(fd, audiobuf + sizeof(AuHeader),
+	if (safe_read(fd, audiobuf + sizeof(AuHeader),
 		 sizeof(VocHeader) - sizeof(AuHeader)) !=
 		 sizeof(VocHeader) - sizeof(AuHeader)) {
 		fprintf(stderr, "%s: read error", command);
@@ -1400,7 +1419,7 @@ static void playback(char *name)
 		goto __end;
 	}
 	/* read 64 bytes for WAVE-header */
-	if (read(fd, audiobuf + sizeof(VocHeader),
+	if (safe_read(fd, audiobuf + sizeof(VocHeader),
 		 64 - sizeof(VocHeader)) !=
 	    64 - sizeof(VocHeader)) {
 		fprintf(stderr, "%s: read error", command);
